# encoding: utf-8


class JustificationHeaderDto(object):
    def __init__(
        self,
        title,      # type: str
        subtitle,   # type: str
    ):
        # type: (...) -> None

        self.title = title
        self.subtitle = subtitle
