class PropertyError(Exception):
    def __init__(self, property_name):
        self.property_name = property_name

    def __str__(self):
        return "Error parsing property {}".format(self.property_name)
