from ._HttpResponseMessage import HttpResponseMessage


class JsonHttpResponseMessage(HttpResponseMessage):
    def __init__(self, status_code, body=None, headers=None):
        if headers is None:
            headers = {}

        if "Content-Type" not in headers:
            headers["Content-Type"] = "application/json; charset=utf-8"

        super(JsonHttpResponseMessage, self).__init__(status_code, body, headers)
