from datetime import datetime
from typing import List

class SaleChallengeItem(object):

    def __init__(self, name, start_date, end_date, products):
        # type: (int, datetime, datetime, List[int]) -> None

        self.name = name
        self.start_date = start_date
        self.end_date = end_date
        self.products = products
