from typing import List


class ListenedEvents(object):
    DISCOUNT_APPLIED = "DISCOUNT_APPLIED"
    ORDER_CANCELED = "ORDER_CANCELED"
    COMMON_CLEAN_DATABASES = "COMMON_CLEAN_DATABASES"
    POS_ALERT_NOTIFICATION_ACK = "PosAlertNotificationAck"
    BOH_CHANGE_NOTIFICATION_CONFIG = "BohChangeNotification"
    BOH_ORDER_PICTURE = "BOH_ORDER_PICTURE"


def all_events():
    # type: (...) -> List[str]

    listened_events = [
        ListenedEvents.DISCOUNT_APPLIED,
        ListenedEvents.ORDER_CANCELED,
        ListenedEvents.COMMON_CLEAN_DATABASES,
        ListenedEvents.POS_ALERT_NOTIFICATION_ACK,
        ListenedEvents.BOH_CHANGE_NOTIFICATION_CONFIG,
        ListenedEvents.BOH_ORDER_PICTURE,
    ]

    return listened_events
