# encoding: utf-8
from typing import Dict


class BenefitsBodyDto(object):
    def __init__(self, orders, type_report):
        # type: (Dict, str) -> ()

        self.items = orders
        self.type_report = type_report
