class SubscriptionInfo(object):
    TYPE_REENTRANT = 1
    TYPE_NON_REENTRANT = 2

    def __init__(self, subject, subscription_type):
        # type: (str, int) -> None
        self.subject = subject
        self.subscription_type = subscription_type
