from typing import Dict


class CustomizationStyle(object):
    def __init__(
            self,
            style   # type: Dict
    ):
        # type: (...) -> CustomizationStyle
        self.style = style
